﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
//using Microsoft.Xna.Framework.Content;

namespace XNAGeometryObject
{
    public class Quad
    {
        public Vector3 Origin;
        public Vector3 UpperLeft;
        public Vector3 LowerLeft;
        public Vector3 UpperRight;
        public Vector3 LowerRight;
        public Vector3 Normal;
        public Vector3 Up;
        public Vector3 Left;

        public VertexPositionNormalTexture[] Vertices { get { return vertices; } }
        public int[] Indexes { get { return indices; } }

        private VertexPositionNormalTexture[] vertices;
        private int[] indices;

        public Quad(Vector3 origin, Vector3 normal, Vector3 up, float width, float height)
        {
            vertices = new VertexPositionNormalTexture[4];
            indices = new int[6];
            Origin = origin;
            Normal = normal;
            Up = up;

            // Calculate the quad corners
            Left = Vector3.Cross( normal, Up );
            Vector3 uppercenter = (Up * height / 2) + origin;
            UpperLeft = uppercenter + (Left * width / 2);
            UpperRight = uppercenter - (Left * width / 2);
            LowerLeft = UpperLeft - (Up * height);
            LowerRight = UpperRight - (Up * height);

            createVertices(new Vector2(0.0f, 0.0f), new Vector2(1.0f, 0.0f), new Vector2(0.0f, 1.0f), new Vector2(1.0f, 1.0f));
        }

        public Quad(Vector3 origin, Vector3 normal, Vector3 up, float width, float height, Vector2 textureUpperLeft, Vector2 textureUpperRight, Vector2 textureLowerLeft, Vector2 textureLowerRight)
        {
            vertices = new VertexPositionNormalTexture[4];
            indices = new int[6];
            Origin = origin;
            Normal = normal;
            Up = up;

            // Calculate the quad corners
            Left = Vector3.Cross(normal, Up);
            Vector3 uppercenter = (Up * height / 2) + origin;
            UpperLeft = uppercenter + (Left * width / 2);
            UpperRight = uppercenter - (Left * width / 2);
            LowerLeft = UpperLeft - (Up * height);
            LowerRight = UpperRight - (Up * height);

            createVertices(textureUpperLeft, textureUpperRight, textureLowerLeft, textureLowerRight);
        }

        private void createVertices(Vector2 textureUpperLeft, Vector2 textureUpperRight, Vector2 textureLowerLeft, Vector2 textureLowerRight)
        {
            // Fill in texture coordinates to display full texture on quad
            //Vector2 textureUpperLeft = new Vector2( 0.0f, 0.0f );
            //Vector2 textureUpperRight = new Vector2( 1.0f, 0.0f );
            //Vector2 textureLowerLeft = new Vector2( 0.0f, 1.0f );
            //Vector2 textureLowerRight = new Vector2( 1.0f, 1.0f );

            // Provide a normal for each vertex
            for (int i = 0; i < vertices.Length; i++)
            {
                vertices[i].Normal = Normal;
            }

            // Set the position and texture coordinate for each
            // vertex
            vertices[0].Position = LowerLeft;
            vertices[0].TextureCoordinate = textureLowerLeft;
            vertices[1].Position = UpperLeft;
            vertices[1].TextureCoordinate = textureUpperLeft;
            vertices[2].Position = LowerRight;
            vertices[2].TextureCoordinate = textureLowerRight;
            vertices[3].Position = UpperRight;
            vertices[3].TextureCoordinate = textureUpperRight;

            // Set the index buffer for each vertex, using
            // clockwise winding
            indices[0] = 0;
            indices[1] = 1;
            indices[2] = 2;
            indices[3] = 2;
            indices[4] = 1;
            indices[5] = 3;
        }
    }
}
